/*
 * Copyright (C) 2008 Luke Kenneth Casson Leighton <lkcl@lkcl.net>
 * Copyright (C) 2008 Alp Toker <alp@atoker.com>
 * Copyright (C) 2008 Apple Inc.
 */

#include "config.h"

#include <wtf/GetPtr.h>
#include "CSSRule.h"
#include "CSSValue.h"
#include "CanvasPixelArray.h"
#include "DOMWindow.h"
#include "Element.h"
#include "Event.h"
#include "GDOMBinding.h"
#include "GDOMObject.h"
#include "HTMLCollection.h"
#include "HTMLImageElement.h"
#include "Node.h"
#include "StyleSheet.h"
#include "Text.h"
#include "gdom/GStringConvert.h"
#include "gdom/GdomHTMLElement.h"
#include "gdom/GdomHTMLImageElement.h"
#include "gdom/PrivateGdomDOMObject.h"
#include "gdom/PrivateGdomHTMLElement.h"
#include "gdom/PrivateGdomHTMLImageElement.h"
#include "gdom/PrivateIntGdomHTMLImageElement.h"

namespace WebCore {
    gpointer toGDOM(Node* node);
    gpointer toGDOM(CSSValue* node);
    gpointer toGDOM(CSSRule* node);
    gpointer toGDOM(Element* node);
    gpointer toGDOM(Document* node);
    gpointer toGDOM(HTMLCollection* node);
    //gpointer toGDOM(SVGPathSeg* node);
    gpointer toGDOM(StyleSheet* node);
    gpointer toGDOM(EventTargetNode* node);
    gpointer toGDOM(EventTarget* node);
    gpointer toGDOM(Event* node);
    gpointer toGDOM(CanvasPixelArray* node);
    gpointer toGDOM(DOMWindow* node);
    gpointer toGDOM(Text* node);
} // namespace WebCore

namespace WebCore {
} /* namespace WebCore */
extern "C" {

GDOMHTMLImageElement::GDOMHTMLImageElement(PassRefPtr<WebCore::HTMLImageElement> impl)
    : GDOMHTMLElement(impl)
{
}

} /* extern "C" */


#include "config.h"
#include "glib-object.h"
#include <webkit/webkitdefines.h>
#include <webkit/webkitprivate.h> 
#include "DOMWindow.h" /* yukk: TODO - make this optional */
#include "Node.h" /* yukk: TODO - make this optional */
#include "NamedNodeMap.h" /* yukk: TODO - make this optional */
#include "CanvasPattern.h" /* yukk: TODO - make this optional */
#include "CanvasGradient.h" /* yukk: TODO - make this optional */
#include "CanvasStyle.h" /* yukk: TODO - make this optional */
#include "CanvasRenderingContext2D.h" /* yukk: TODO - make this optional */
#include <gdom/PrivateGdomDOMObject.h> /* yukk: TODO - make this optional */
#include <gdom/GdomEventTargetNode.h> /* yukk: TODO - make this optional */
#include <gdom/PrivateGdomEventTargetNode.h> /* yukk: TODO - make this optional on EventTarget*/
#include "HTMLImageElement.h"
#include <gdom/GdomHTMLImageElement.h>
#include <gdom/PrivateGdomHTMLImageElement.h>
#include <gdom/PrivateIntGdomHTMLImageElement.h>
#include <gdom/GdomHTMLImageElementprops.h>

#include "gdom/GStringConvert.h"

#include "CanvasPixelArray.h"
#include "Event.h"
#include "Text.h"
#include "CSSValue.h"
#include "CSSRule.h"
#include "HTMLCollection.h"
#include "StyleSheet.h"
#include "Document.h"
#include "webkitmarshal.h"
#include <gdom/GdomEvent.h>



namespace WebCore {
    gpointer toGDOM(Node* node);
    gpointer toGDOM(CSSValue* node);
    gpointer toGDOM(CSSRule* node);
    gpointer toGDOM(Element* node);
    gpointer toGDOM(Document* node);
    gpointer toGDOM(HTMLCollection* node);
    //gpointer toGDOM(SVGPathSeg* node);
    gpointer toGDOM(StyleSheet* node);
    gpointer toGDOM(EventTargetNode* node);
    gpointer toGDOM(EventTarget* node);
    gpointer toGDOM(Event* node);
    gpointer toGDOM(CanvasPixelArray* node);
    gpointer toGDOM(DOMWindow* node);
    gpointer toGDOM(Text* node);
} // namespace WebCore



G_BEGIN_DECLS

G_DEFINE_TYPE(GdomHTMLImageElement, gdom_html_image_element, GDOM_TYPE_HTML_ELEMENT);

/*
struct _GdomHTMLImageElementPrivate {
    class WebCore::HTMLImageElement *item;
};

#define GDOM_HTML_IMAGE_ELEMENT_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE((obj), GDOM_TYPE_HTML_IMAGE_ELEMENT, GdomHTMLImageElementPrivate))
*/
G_END_DECLS
WebCore::HTMLImageElement *
gdom_html_image_element_get_private(GdomHTMLImageElement *request)
{
    GDOMHTMLImageElement *res;
    res = static_cast<GDOMHTMLImageElement*>(gdom_dom_object_get_private(GDOM_DOM_OBJECT(request)));
    return static_cast<WebCore::HTMLImageElement*>(res->impl());
}

G_BEGIN_DECLS

enum {
    BROWSER_EVENT,
    LAST_SIGNAL
};

static guint event_signals[LAST_SIGNAL] = { 0, };

enum {
    PROP_0,
    PROP_NAME,
    PROP_ALIGN,
    PROP_ALT,
    PROP_BORDER,
    PROP_HEIGHT,
    PROP_HSPACE,
    PROP_IS_MAP,
    PROP_LONG_DESC,
    PROP_SRC,
    PROP_USE_MAP,
    PROP_VSPACE,
    PROP_WIDTH,
    PROP_COMPLETE,
    PROP_LOWSRC,
    PROP_NATURAL_HEIGHT,
    PROP_NATURAL_WIDTH,
    PROP_X,
    PROP_Y,
};


static void gdom_html_image_element_finalize(GObject* object)
{
    /*GdomHTMLImageElement* request = GDOM_HTML_IMAGE_ELEMENT(object);*/
    /*GdomHTMLImageElementPrivate* priv = request->priv;*/

    /*GObjectClass *parent_class = g_type_class_peek (g_type_peek_parent (GDOM_TYPE_HTML_ELEMENT));*/

    /* Free any memory or close any files */

    /*parent_class-> finalize (object);*/

    G_OBJECT_CLASS(gdom_html_image_element_parent_class)->finalize(object);
}

static void gdom_html_image_element__set_property_fn(GObject * object, guint prop_id,
                     const GValue * value,
                     GParamSpec * pspec)
{
    GdomHTMLImageElement *self = GDOM_HTML_IMAGE_ELEMENT(object);
    WebCore::HTMLImageElement * gself = (gdom_html_image_element_get_private(self));
    switch (prop_id) {
     case PROP_NAME:
    {
         gself->setName(WebCore::String::fromUTF8(g_value_get_string (value)) );
         break;
    }
     case PROP_ALIGN:
    {
         gself->setAlign(WebCore::String::fromUTF8(g_value_get_string (value)) );
         break;
    }
     case PROP_ALT:
    {
         gself->setAlt(WebCore::String::fromUTF8(g_value_get_string (value)) );
         break;
    }
     case PROP_BORDER:
    {
         gself->setBorder(WebCore::String::fromUTF8(g_value_get_string (value)) );
         break;
    }
     case PROP_HEIGHT:
    {
         gself->setHeight((g_value_get_long (value)) );
         break;
    }
     case PROP_HSPACE:
    {
         gself->setHspace((g_value_get_long (value)) );
         break;
    }
     case PROP_IS_MAP:
    {
         gself->setIsMap((g_value_get_boolean (value)) );
         break;
    }
     case PROP_LONG_DESC:
    {
         gself->setLongDesc(WebCore::String::fromUTF8(g_value_get_string (value)) );
         break;
    }
     case PROP_SRC:
    {
         gself->setSrc(WebCore::String::fromUTF8(g_value_get_string (value)) );
         break;
    }
     case PROP_USE_MAP:
    {
         gself->setUseMap(WebCore::String::fromUTF8(g_value_get_string (value)) );
         break;
    }
     case PROP_VSPACE:
    {
         gself->setVspace((g_value_get_long (value)) );
         break;
    }
     case PROP_WIDTH:
    {
         gself->setWidth((g_value_get_long (value)) );
         break;
    }
     case PROP_LOWSRC:
    {
         gself->setLowsrc(WebCore::String::fromUTF8(g_value_get_string (value)) );
         break;
    }
    default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
        break;
    }
}


static void gdom_html_image_element__get_property_fn(GObject * object, guint prop_id,
                     GValue * value,
                     GParamSpec * pspec)
{
    GdomHTMLImageElement *self = GDOM_HTML_IMAGE_ELEMENT(object);
    WebCore::HTMLImageElement * gself = (gdom_html_image_element_get_private(self));
    switch (prop_id) {
    case PROP_NAME:
    {
         g_value_set_string (value, _G_GET_UTF8(gself->name()));
         break;
    }
    case PROP_ALIGN:
    {
         g_value_set_string (value, _G_GET_UTF8(gself->align()));
         break;
    }
    case PROP_ALT:
    {
         g_value_set_string (value, _G_GET_UTF8(gself->alt()));
         break;
    }
    case PROP_BORDER:
    {
         g_value_set_string (value, _G_GET_UTF8(gself->border()));
         break;
    }
    case PROP_HEIGHT:
    {
         g_value_set_long (value, gself->height());
         break;
    }
    case PROP_HSPACE:
    {
         g_value_set_long (value, gself->hspace());
         break;
    }
    case PROP_IS_MAP:
    {
         g_value_set_boolean (value, gself->isMap());
         break;
    }
    case PROP_LONG_DESC:
    {
         g_value_set_string (value, _G_GET_UTF8(gself->longDesc()));
         break;
    }
    case PROP_SRC:
    {
         g_value_set_string (value, _G_GET_UTF8(gself->src()));
         break;
    }
    case PROP_USE_MAP:
    {
         g_value_set_string (value, _G_GET_UTF8(gself->useMap()));
         break;
    }
    case PROP_VSPACE:
    {
         g_value_set_long (value, gself->vspace());
         break;
    }
    case PROP_WIDTH:
    {
         g_value_set_long (value, gself->width());
         break;
    }
    case PROP_COMPLETE:
    {
         g_value_set_boolean (value, gself->complete());
         break;
    }
    case PROP_LOWSRC:
    {
         g_value_set_string (value, _G_GET_UTF8(gself->lowsrc()));
         break;
    }
    case PROP_NATURAL_HEIGHT:
    {
         g_value_set_long (value, gself->naturalHeight());
         break;
    }
    case PROP_NATURAL_WIDTH:
    {
         g_value_set_long (value, gself->naturalWidth());
         break;
    }
    case PROP_X:
    {
         g_value_set_long (value, gself->x());
         break;
    }
    case PROP_Y:
    {
         g_value_set_long (value, gself->y());
         break;
    }
    default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
        break;
    }
}


static void gdom_html_image_element_class_init(GdomHTMLImageElementClass* requestClass)
{
    GObjectClass *g_klass = G_OBJECT_CLASS (requestClass);
    g_klass->finalize = gdom_html_image_element_finalize;
    g_klass->set_property = gdom_html_image_element__set_property_fn;
    g_klass->get_property = gdom_html_image_element__get_property_fn;

    g_object_class_install_property (g_klass, PROP_NAME,
            g_param_spec_string ("name", /* name */
                "html_image_element_name", /* short description */
                "read-write  gchar * HTMLImageElement.name", /* longer - could do with some extra doc stuff here */
                 "", /* default */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_ALIGN,
            g_param_spec_string ("align", /* name */
                "html_image_element_align", /* short description */
                "read-write  gchar * HTMLImageElement.align", /* longer - could do with some extra doc stuff here */
                 "", /* default */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_ALT,
            g_param_spec_string ("alt", /* name */
                "html_image_element_alt", /* short description */
                "read-write  gchar * HTMLImageElement.alt", /* longer - could do with some extra doc stuff here */
                 "", /* default */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_BORDER,
            g_param_spec_string ("border", /* name */
                "html_image_element_border", /* short description */
                "read-write  gchar * HTMLImageElement.border", /* longer - could do with some extra doc stuff here */
                 "", /* default */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_HEIGHT,
            g_param_spec_long ("height", /* name */
                "html_image_element_height", /* short description */
                "read-write  glong HTMLImageElement.height", /* longer - could do with some extra doc stuff here */
                 G_MINLONG,        /* min */
                G_MAXLONG, /* max */
                0,        /* default */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_HSPACE,
            g_param_spec_long ("hspace", /* name */
                "html_image_element_hspace", /* short description */
                "read-write  glong HTMLImageElement.hspace", /* longer - could do with some extra doc stuff here */
                 G_MINLONG,        /* min */
                G_MAXLONG, /* max */
                0,        /* default */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_IS_MAP,
            g_param_spec_boolean ("is_map", /* name */
                "html_image_element_is_map", /* short description */
                "read-write  gboolean HTMLImageElement.is_map", /* longer - could do with some extra doc stuff here */
                 FALSE, /* default */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_LONG_DESC,
            g_param_spec_string ("long_desc", /* name */
                "html_image_element_long_desc", /* short description */
                "read-write  gchar * HTMLImageElement.long_desc", /* longer - could do with some extra doc stuff here */
                 "", /* default */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_SRC,
            g_param_spec_string ("src", /* name */
                "html_image_element_src", /* short description */
                "read-write  gchar * HTMLImageElement.src", /* longer - could do with some extra doc stuff here */
                 "", /* default */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_USE_MAP,
            g_param_spec_string ("use_map", /* name */
                "html_image_element_use_map", /* short description */
                "read-write  gchar * HTMLImageElement.use_map", /* longer - could do with some extra doc stuff here */
                 "", /* default */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_VSPACE,
            g_param_spec_long ("vspace", /* name */
                "html_image_element_vspace", /* short description */
                "read-write  glong HTMLImageElement.vspace", /* longer - could do with some extra doc stuff here */
                 G_MINLONG,        /* min */
                G_MAXLONG, /* max */
                0,        /* default */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_WIDTH,
            g_param_spec_long ("width", /* name */
                "html_image_element_width", /* short description */
                "read-write  glong HTMLImageElement.width", /* longer - could do with some extra doc stuff here */
                 G_MINLONG,        /* min */
                G_MAXLONG, /* max */
                0,        /* default */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_COMPLETE,
            g_param_spec_boolean ("complete", /* name */
                "html_image_element_complete", /* short description */
                "read-only  gboolean HTMLImageElement.complete", /* longer - could do with some extra doc stuff here */
                 FALSE, /* default */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_LOWSRC,
            g_param_spec_string ("lowsrc", /* name */
                "html_image_element_lowsrc", /* short description */
                "read-write  gchar * HTMLImageElement.lowsrc", /* longer - could do with some extra doc stuff here */
                 "", /* default */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_NATURAL_HEIGHT,
            g_param_spec_long ("natural_height", /* name */
                "html_image_element_natural_height", /* short description */
                "read-only  glong HTMLImageElement.natural_height", /* longer - could do with some extra doc stuff here */
                 G_MINLONG,        /* min */
                G_MAXLONG, /* max */
                0,        /* default */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_NATURAL_WIDTH,
            g_param_spec_long ("natural_width", /* name */
                "html_image_element_natural_width", /* short description */
                "read-only  glong HTMLImageElement.natural_width", /* longer - could do with some extra doc stuff here */
                 G_MINLONG,        /* min */
                G_MAXLONG, /* max */
                0,        /* default */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_X,
            g_param_spec_long ("x", /* name */
                "html_image_element_x", /* short description */
                "read-only  glong HTMLImageElement.x", /* longer - could do with some extra doc stuff here */
                 G_MINLONG,        /* min */
                G_MAXLONG, /* max */
                0,        /* default */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_Y,
            g_param_spec_long ("y", /* name */
                "html_image_element_y", /* short description */
                "read-only  glong HTMLImageElement.y", /* longer - could do with some extra doc stuff here */
                 G_MINLONG,        /* min */
                G_MAXLONG, /* max */
                0,        /* default */
                 (GParamFlags)G_PARAM_READABLE));





    /*g_type_class_remove_private(requestClass, sizeof(GdomHTMLImageElementPrivate));*/
}

static void gdom_html_image_element_init(GdomHTMLImageElement* request)
{
    /*GdomHTMLImageElementPrivate* priv = GDOM_HTML_IMAGE_ELEMENT_GET_PRIVATE(request);
    request->priv = priv;*/
}

} /* extern "C" */

